#!/usr/bin/python3

import os

import pygame.mixer
import pygame.event
from .track import Track
from .track_progress_timer import TrackProgress


class Player:
    def __init__(self, app, file_paths: list=[]):
        self.app = app
        self.track_progress = TrackProgress(self.app)

        pygame.mixer.init()
        self.mixer = pygame.mixer
        self.music_channel = self.mixer.Channel(0)

        self.playing = False
        self.paused = False
        self.searched = False

        if not file_paths:
            self.current_playlist = []
            # loading of last opened files will be implemented in the future

        else:
            self.current_playlist = self.load_tracks(file_paths)

        if self.current_playlist:
            self.playing_track = self.current_playlist[0]
            self.current_playlist_index = 0
            self.playing_segment = self.playing_track.sound
            self.playing_segment_duration = self.playing_track.duration

        else:
            self.playing_track = None
            self.current_playlist_index = 0
            self.playing_segment = None
            self.playing_segment_duration = 0

    def load_tracks(self, track_paths: list[str]):
        """
        Load tracks from list of paths.
        """

        tracks = []

        for track_path in track_paths:
            if os.path.isfile(track_path):
                tracks.append(Track(track_path, True))

        return tracks

    def track_finished(self):
        self.current_playlist_index += 1
        if self.current_playlist_index < len(self.current_playlist):
            self.playing_track = self.current_playlist[self.current_playlist_index]
            self.playing_segment = self.playing_track.sound
            self.playing_segment_duration = self.playing_track.duration

            self.music_channel.play(self.playing_track.sound)
            self.track_progress.start()
            self.app.gui_communication.on_track_start()

        else:
            self.current_playlist_index -= 1
            self.stop()

    def start_playing(self):
        self.music_channel.play(self.playing_track.sound)
        self.track_progress.start()
        self.paused = False
        self.playing = True

    def pause(self):
        self.music_channel.pause()
        self.track_progress.pause()
        self.paused = True

    def unpause(self):
        self.music_channel.unpause()
        self.track_progress.unpause()
        self.paused = False

    def skip_current(self):
        self.music_channel.stop()
        self.track_progress.stop()
        self.track_finished()

    def previous_track(self):
        if self.current_playlist_index > 0:
            self.music_channel.stop()

            self.current_playlist_index -= 1
            self.playing_track = self.current_playlist[self.current_playlist_index]
            self.track_progress.stop()

            self.playing_segment = self.playing_track.sound
            self.playing_segment_duration = self.playing_track.duration

            self.music_channel.play(self.playing_track.sound)
            self.track_progress.start()
            self.app.gui_communication.on_track_start()

    def toggle_playing(self):
        if self.playing and self.paused:
            self.unpause()

        elif self.playing:
            self.pause()

        else:
            self.start_playing()

    def stop(self):
        self.music_channel.stop()
        self.track_progress.stop()
        self.playing = False
        self.playing_segment_duration = self.playing_track.duration

    def seek(self, position: int):
        self.searched = True
        self.music_channel.stop()
        self.track_progress.stop()
        (self.playing_segment, self.playing_segment_duration) = self.playing_track.remaining(position)
        self.music_channel.play(self.playing_segment)
        self.track_progress.start()

